import os
import cffi
import logging
from cornice.util import to_list
from pyclibrary.utils import add_header_locations

logger = logging.getLogger(__name__)

class LibraryAdapterCFFI(object):

    def __init__(self, header_files, library_file, include_path=None, library_path=None, cache_path=None):
        self.header_files = to_list(header_files)
        self.library_file = library_file
        self.include_path = include_path or os.curdir
        self.library_path = library_path or os.curdir
        self.cache_path = cache_path or './var'

        cache_key = u'_'.join(self.header_files) + '.pyclibrary'
        self.cache_file = os.path.join(self.cache_path, cache_key)

        logger.info('Setting up library "{}" with headers "{}"'.format(self.library_file, ', '.join(self.header_files)))

        self.setup()
        self.parse()
        self.load()

    def setup(self):

        self.ffi = cffi.FFI()
        print self.include_path
        self.ffi.set_source('h2m_lst', None, include_dirs=[self.include_path], library_dirs=[self.library_path])
        return

        """
        # counter "ValueError: number of bits invalid for bit field"
        monkeypatch_pyclibrary_ctypes_struct()

        # register header- and library paths
        # https://pyclibrary.readthedocs.org/en/latest/get_started/configuration.html#specifying-headers-and-libraries-locations
        #curr_dir = os.path.dirname(__file__)
        curr_dir = os.curdir
        # TODO: this probably acts on a global basis; think about it
        add_header_locations([self.include_path])
        add_library_locations([self.library_path])

        # define extra types suitable for embedded use
        types = {
            'uint8_t': c_uint8,
            'uint16_t': c_uint16,
            'uint32_t': c_uint32,
            'int8_t': c_int8,
            'int16_t': c_int16,
            'int32_t': c_int32,
            }
        auto_init(extra_types=types)
        """

    def parse(self):
        # use an improved CParser which can grok/skip more constructor flavours
        #self.parser = CParserEnhanced(self.header_files, cache=self.cache_file)
        header_file = os.path.join(self.include_path, self.header_files[0])
        with open(header_file) as f_headers:
            payload = f_headers.read().replace('\r', '\n')
            #print 'payload:', payload
            self.ffi.cdef(payload)
        #self.ffi.compile()

    def load(self):
        self.clib = CLibrary(self.library_file, self.parser, prefix='Lib_', lock_calls=False, convention='cdll', backend='ctypes')

    def struct_names(self):
        return self.parser.defs['structs'].keys()
